import httpx
import json
import pathlib
from . import app
from flask import request, abort

BASE_API_URL = "https://scrapple.fly.dev/hearings"
BASE_DIR = pathlib.Path("data/congress-hearing-118-house")
# not actually secret, but for testing API Keys
API_SECRET = "53cR3T"


def _api_key_check():
    key = request.args.get("api_key", None)
    if not key:
        abort(401, "Must provide api_key parameter.")
    pieces = key.split("-", 1)
    if len(pieces) != 2 or pieces[1] != API_SECRET:
        abort(401, "API Key should be in format '{username}-{secret}'")


def _shortened(hearing):
    """return compact hearing form for list view"""
    return {
        "chamber": hearing["request"]["chamber"],
        "congress": hearing["request"]["congress"],
        "jacketNumber": hearing["request"]["jacketNumber"],
        "updateDate": hearing["hearing"]["updateDate"],
        "url": BASE_API_URL + "/" + hearing["request"]["jacketNumber"],
    }


def _load_hearings():
    all = {}
    for hfile in BASE_DIR.glob("*.json"):
        data = json.loads(hfile.read_text())
        all[data["request"]["jacketNumber"]] = data
    return all


ALL_HEARINGS = _load_hearings()
HEARING_IDS = list(ALL_HEARINGS.keys())  # semi-consistent ordering


@app.route("/hearings")
def hearings_list():
    _api_key_check()
    offset = int(request.args.get("offset", 0))
    limit = min(int(request.args.get("limit", 50)), 200)
    format = request.args.get("format", "json")
    if offset < 0:
        abort(404)
    if offset + limit > len(ALL_HEARINGS):
        next_url = None
    else:
        next_url = str(
            httpx.URL(BASE_API_URL).copy_with(
                params={"offset": offset + limit, "limit": limit, "format": format}
            )
        )
    hearing_ids = HEARING_IDS[offset : offset + limit]
    hearings = [ALL_HEARINGS[h] for h in hearing_ids]
    return {
        "hearings": [_shortened(h) for h in hearings],
        "pagination": {"count": len(ALL_HEARINGS), "next": next_url},
        "chamber": {
            "chamber": "house",
            "congress": "118",
            "contentType": "application/json",
            "format": "json",
        },
    }


@app.route("/hearings/<jacket_id>")
def hearings_detail(jacket_id):
    _api_key_check()
    if jacket_id not in ALL_HEARINGS:
        abort(404)
    return ALL_HEARINGS[jacket_id]
